# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: reports.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\rreports.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xa5\x14\n\x05Trace\x12.\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0b\x64uration_ns\x18\x0b \x01(\x04\x12\x19\n\x04root\x18\x0e \x01(\x0b\x32\x0b.Trace.Node\x12\x11\n\tsignature\x18\x13 \x01(\t\x12\x1f\n\x17unexecutedOperationBody\x18\x1b \x01(\t\x12\x1f\n\x17unexecutedOperationName\x18\x1c \x01(\t\x12\x1f\n\x07\x64\x65tails\x18\x06 \x01(\x0b\x32\x0e.Trace.Details\x12\x13\n\x0b\x63lient_name\x18\x07 \x01(\t\x12\x16\n\x0e\x63lient_version\x18\x08 \x01(\t\x12\x19\n\x04http\x18\n \x01(\x0b\x32\x0b.Trace.HTTP\x12(\n\x0c\x63\x61\x63he_policy\x18\x12 \x01(\x0b\x32\x12.Trace.CachePolicy\x12(\n\nquery_plan\x18\x1a \x01(\x0b\x32\x14.Trace.QueryPlanNode\x12\x1c\n\x14\x66ull_query_cache_hit\x18\x14 \x01(\x08\x12\x1b\n\x13persisted_query_hit\x18\x15 \x01(\x08\x12 \n\x18persisted_query_register\x18\x16 \x01(\x08\x12\x1c\n\x14registered_operation\x18\x18 \x01(\x08\x12\x1b\n\x13\x66orbidden_operation\x18\x19 \x01(\x08\x12\x1e\n\x16\x66ield_execution_weight\x18\x1f \x01(\x01\x1ay\n\x0b\x43\x61\x63hePolicy\x12\'\n\x05scope\x18\x01 \x01(\x0e\x32\x18.Trace.CachePolicy.Scope\x12\x12\n\nmax_age_ns\x18\x02 \x01(\x03"-\n\x05Scope\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06PUBLIC\x10\x01\x12\x0b\n\x07PRIVATE\x10\x02\x1a\x92\x01\n\x07\x44\x65tails\x12\x39\n\x0evariables_json\x18\x04 \x03(\x0b\x32!.Trace.Details.VariablesJsonEntry\x12\x16\n\x0eoperation_name\x18\x03 \x01(\t\x1a\x34\n\x12VariablesJsonEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aZ\n\x05\x45rror\x12\x0f\n\x07message\x18\x01 \x01(\t\x12!\n\x08location\x18\x02 \x03(\x0b\x32\x0f.Trace.Location\x12\x0f\n\x07time_ns\x18\x03 \x01(\x04\x12\x0c\n\x04json\x18\x04 \x01(\t\x1a\x9c\x04\n\x04HTTP\x12"\n\x06method\x18\x01 \x01(\x0e\x32\x12.Trace.HTTP.Method\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x0c\n\x04path\x18\x03 \x01(\t\x12\x38\n\x0frequest_headers\x18\x04 \x03(\x0b\x32\x1f.Trace.HTTP.RequestHeadersEntry\x12:\n\x10response_headers\x18\x05 \x03(\x0b\x32 .Trace.HTTP.ResponseHeadersEntry\x12\x13\n\x0bstatus_code\x18\x06 \x01(\r\x12\x0e\n\x06secure\x18\x08 \x01(\x08\x12\x10\n\x08protocol\x18\t \x01(\t\x1a\x17\n\x06Values\x12\r\n\x05value\x18\x01 \x03(\t\x1aI\n\x13RequestHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.Trace.HTTP.Values:\x02\x38\x01\x1aJ\n\x14ResponseHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.Trace.HTTP.Values:\x02\x38\x01"w\n\x06Method\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07OPTIONS\x10\x01\x12\x07\n\x03GET\x10\x02\x12\x08\n\x04HEAD\x10\x03\x12\x08\n\x04POST\x10\x04\x12\x07\n\x03PUT\x10\x05\x12\n\n\x06\x44\x45LETE\x10\x06\x12\t\n\x05TRACE\x10\x07\x12\x0b\n\x07\x43ONNECT\x10\x08\x12\t\n\x05PATCH\x10\t\x1a(\n\x08Location\x12\x0c\n\x04line\x18\x01 \x01(\r\x12\x0e\n\x06\x63olumn\x18\x02 \x01(\r\x1a\x85\x02\n\x04Node\x12\x17\n\rresponse_name\x18\x01 \x01(\tH\x00\x12\x0f\n\x05index\x18\x02 \x01(\rH\x00\x12\x1b\n\x13original_field_name\x18\x0e \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x13\n\x0bparent_type\x18\r \x01(\t\x12(\n\x0c\x63\x61\x63he_policy\x18\x05 \x01(\x0b\x32\x12.Trace.CachePolicy\x12\x12\n\nstart_time\x18\x08 \x01(\x04\x12\x10\n\x08\x65nd_time\x18\t \x01(\x04\x12\x1b\n\x05\x65rror\x18\x0b \x03(\x0b\x32\x0c.Trace.Error\x12\x1a\n\x05\x63hild\x18\x0c \x03(\x0b\x32\x0b.Trace.NodeB\x04\n\x02idJ\x04\x08\x04\x10\x05\x1a\xe2\x05\n\rQueryPlanNode\x12\x35\n\x08sequence\x18\x01 \x01(\x0b\x32!.Trace.QueryPlanNode.SequenceNodeH\x00\x12\x35\n\x08parallel\x18\x02 \x01(\x0b\x32!.Trace.QueryPlanNode.ParallelNodeH\x00\x12/\n\x05\x66\x65tch\x18\x03 \x01(\x0b\x32\x1e.Trace.QueryPlanNode.FetchNodeH\x00\x12\x33\n\x07\x66latten\x18\x04 \x01(\x0b\x32 .Trace.QueryPlanNode.FlattenNodeH\x00\x1a\x33\n\x0cSequenceNode\x12#\n\x05nodes\x18\x01 \x03(\x0b\x32\x14.Trace.QueryPlanNode\x1a\x33\n\x0cParallelNode\x12#\n\x05nodes\x18\x01 \x03(\x0b\x32\x14.Trace.QueryPlanNode\x1a\xd2\x01\n\tFetchNode\x12\x14\n\x0cservice_name\x18\x01 \x01(\t\x12\x1c\n\x14trace_parsing_failed\x18\x02 \x01(\x08\x12\x15\n\x05trace\x18\x03 \x01(\x0b\x32\x06.Trace\x12\x18\n\x10sent_time_offset\x18\x04 \x01(\x04\x12-\n\tsent_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x31\n\rreceived_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1ar\n\x0b\x46lattenNode\x12?\n\rresponse_path\x18\x01 \x03(\x0b\x32(.Trace.QueryPlanNode.ResponsePathElement\x12"\n\x04node\x18\x02 \x01(\x0b\x32\x14.Trace.QueryPlanNode\x1a\x42\n\x13ResponsePathElement\x12\x14\n\nfield_name\x18\x01 \x01(\tH\x00\x12\x0f\n\x05index\x18\x02 \x01(\rH\x00\x42\x04\n\x02idB\x06\n\x04nodeJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\t\x10\nJ\x04\x08\x0c\x10\rJ\x04\x08\r\x10\x0eJ\x04\x08\x17\x10\x18"\xaf\x01\n\x0cReportHeader\x12\x11\n\tgraph_ref\x18\x0c \x01(\t\x12\x10\n\x08hostname\x18\x05 \x01(\t\x12\x15\n\ragent_version\x18\x06 \x01(\t\x12\x17\n\x0fservice_version\x18\x07 \x01(\t\x12\x17\n\x0fruntime_version\x18\x08 \x01(\t\x12\r\n\x05uname\x18\t \x01(\t\x12\x1c\n\x14\x65xecutable_schema_id\x18\x0b \x01(\tJ\x04\x08\x03\x10\x04"\xbd\x01\n\x0ePathErrorStats\x12/\n\x08\x63hildren\x18\x01 \x03(\x0b\x32\x1d.PathErrorStats.ChildrenEntry\x12\x14\n\x0c\x65rrors_count\x18\x04 \x01(\x04\x12"\n\x1arequests_with_errors_count\x18\x05 \x01(\x04\x1a@\n\rChildrenEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.PathErrorStats:\x02\x38\x01"\xcf\x03\n\x11QueryLatencyStats\x12\x15\n\rlatency_count\x18\r \x03(\x12\x12\x15\n\rrequest_count\x18\x02 \x01(\x04\x12\x12\n\ncache_hits\x18\x03 \x01(\x04\x12\x1c\n\x14persisted_query_hits\x18\x04 \x01(\x04\x12\x1e\n\x16persisted_query_misses\x18\x05 \x01(\x04\x12\x1b\n\x13\x63\x61\x63he_latency_count\x18\x0e \x03(\x12\x12)\n\x10root_error_stats\x18\x07 \x01(\x0b\x32\x0f.PathErrorStats\x12"\n\x1arequests_with_errors_count\x18\x08 \x01(\x04\x12\x1e\n\x16public_cache_ttl_count\x18\x0f \x03(\x12\x12\x1f\n\x17private_cache_ttl_count\x18\x10 \x03(\x12\x12"\n\x1aregistered_operation_count\x18\x0b \x01(\x04\x12!\n\x19\x66orbidden_operation_count\x18\x0c \x01(\x04\x12.\n&requests_without_field_instrumentation\x18\x11 \x01(\x04J\x04\x08\x01\x10\x02J\x04\x08\x06\x10\x07J\x04\x08\t\x10\nJ\x04\x08\n\x10\x0b"A\n\x0cStatsContext\x12\x13\n\x0b\x63lient_name\x18\x02 \x01(\t\x12\x16\n\x0e\x63lient_version\x18\x03 \x01(\tJ\x04\x08\x01\x10\x02"r\n\x1f\x43ontextualizedQueryLatencyStats\x12/\n\x13query_latency_stats\x18\x01 \x01(\x0b\x32\x12.QueryLatencyStats\x12\x1e\n\x07\x63ontext\x18\x02 \x01(\x0b\x32\r.StatsContext"\xba\x01\n\x17\x43ontextualizedTypeStats\x12\x1e\n\x07\x63ontext\x18\x01 \x01(\x0b\x32\r.StatsContext\x12@\n\rper_type_stat\x18\x02 \x03(\x0b\x32).ContextualizedTypeStats.PerTypeStatEntry\x1a=\n\x10PerTypeStatEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x18\n\x05value\x18\x02 \x01(\x0b\x32\t.TypeStat:\x02\x38\x01"\xce\x01\n\tFieldStat\x12\x13\n\x0breturn_type\x18\x03 \x01(\t\x12\x14\n\x0c\x65rrors_count\x18\x04 \x01(\x04\x12 \n\x18observed_execution_count\x18\x05 \x01(\x04\x12!\n\x19\x65stimated_execution_count\x18\n \x01(\x04\x12"\n\x1arequests_with_errors_count\x18\x06 \x01(\x04\x12\x15\n\rlatency_count\x18\t \x03(\x12J\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\t"\x8c\x01\n\x08TypeStat\x12\x33\n\x0eper_field_stat\x18\x03 \x03(\x0b\x32\x1b.TypeStat.PerFieldStatEntry\x1a?\n\x11PerFieldStatEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x19\n\x05value\x18\x02 \x01(\x0b\x32\n.FieldStat:\x02\x38\x01J\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03"D\n\x17ReferencedFieldsForType\x12\x13\n\x0b\x66ield_names\x18\x01 \x03(\t\x12\x14\n\x0cis_interface\x18\x02 \x01(\x08"\xd4\x01\n\x06Report\x12\x1d\n\x06header\x18\x01 \x01(\x0b\x32\r.ReportHeader\x12\x35\n\x10traces_per_query\x18\x05 \x03(\x0b\x32\x1b.Report.TracesPerQueryEntry\x12,\n\x08\x65nd_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\x46\n\x13TracesPerQueryEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.TracesAndStats:\x02\x38\x01"\xe3\x01\n\x13\x43ontextualizedStats\x12\x1e\n\x07\x63ontext\x18\x01 \x01(\x0b\x32\r.StatsContext\x12/\n\x13query_latency_stats\x18\x02 \x01(\x0b\x32\x12.QueryLatencyStats\x12<\n\rper_type_stat\x18\x03 \x03(\x0b\x32%.ContextualizedStats.PerTypeStatEntry\x1a=\n\x10PerTypeStatEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x18\n\x05value\x18\x02 \x01(\x0b\x32\t.TypeStat:\x02\x38\x01"\xb9\x02\n\x0eTracesAndStats\x12\x15\n\x05trace\x18\x01 \x03(\x0b\x32\x06.Trace\x12\x30\n\x12stats_with_context\x18\x02 \x03(\x0b\x32\x14.ContextualizedStats\x12N\n\x19referenced_fields_by_type\x18\x04 \x03(\x0b\x32+.TracesAndStats.ReferencedFieldsByTypeEntry\x12\x35\n%internal_traces_contributing_to_stats\x18\x03 \x03(\x0b\x32\x06.Trace\x1aW\n\x1bReferencedFieldsByTypeEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.ReferencedFieldsForType:\x02\x38\x01\x62\x06proto3'
)


_TRACE = DESCRIPTOR.message_types_by_name["Trace"]
_TRACE_CACHEPOLICY = _TRACE.nested_types_by_name["CachePolicy"]
_TRACE_DETAILS = _TRACE.nested_types_by_name["Details"]
_TRACE_DETAILS_VARIABLESJSONENTRY = _TRACE_DETAILS.nested_types_by_name[
    "VariablesJsonEntry"
]
_TRACE_ERROR = _TRACE.nested_types_by_name["Error"]
_TRACE_HTTP = _TRACE.nested_types_by_name["HTTP"]
_TRACE_HTTP_VALUES = _TRACE_HTTP.nested_types_by_name["Values"]
_TRACE_HTTP_REQUESTHEADERSENTRY = _TRACE_HTTP.nested_types_by_name[
    "RequestHeadersEntry"
]
_TRACE_HTTP_RESPONSEHEADERSENTRY = _TRACE_HTTP.nested_types_by_name[
    "ResponseHeadersEntry"
]
_TRACE_LOCATION = _TRACE.nested_types_by_name["Location"]
_TRACE_NODE = _TRACE.nested_types_by_name["Node"]
_TRACE_QUERYPLANNODE = _TRACE.nested_types_by_name["QueryPlanNode"]
_TRACE_QUERYPLANNODE_SEQUENCENODE = _TRACE_QUERYPLANNODE.nested_types_by_name[
    "SequenceNode"
]
_TRACE_QUERYPLANNODE_PARALLELNODE = _TRACE_QUERYPLANNODE.nested_types_by_name[
    "ParallelNode"
]
_TRACE_QUERYPLANNODE_FETCHNODE = _TRACE_QUERYPLANNODE.nested_types_by_name["FetchNode"]
_TRACE_QUERYPLANNODE_FLATTENNODE = _TRACE_QUERYPLANNODE.nested_types_by_name[
    "FlattenNode"
]
_TRACE_QUERYPLANNODE_RESPONSEPATHELEMENT = _TRACE_QUERYPLANNODE.nested_types_by_name[
    "ResponsePathElement"
]
_REPORTHEADER = DESCRIPTOR.message_types_by_name["ReportHeader"]
_PATHERRORSTATS = DESCRIPTOR.message_types_by_name["PathErrorStats"]
_PATHERRORSTATS_CHILDRENENTRY = _PATHERRORSTATS.nested_types_by_name["ChildrenEntry"]
_QUERYLATENCYSTATS = DESCRIPTOR.message_types_by_name["QueryLatencyStats"]
_STATSCONTEXT = DESCRIPTOR.message_types_by_name["StatsContext"]
_CONTEXTUALIZEDQUERYLATENCYSTATS = DESCRIPTOR.message_types_by_name[
    "ContextualizedQueryLatencyStats"
]
_CONTEXTUALIZEDTYPESTATS = DESCRIPTOR.message_types_by_name["ContextualizedTypeStats"]
_CONTEXTUALIZEDTYPESTATS_PERTYPESTATENTRY = (
    _CONTEXTUALIZEDTYPESTATS.nested_types_by_name["PerTypeStatEntry"]
)
_FIELDSTAT = DESCRIPTOR.message_types_by_name["FieldStat"]
_TYPESTAT = DESCRIPTOR.message_types_by_name["TypeStat"]
_TYPESTAT_PERFIELDSTATENTRY = _TYPESTAT.nested_types_by_name["PerFieldStatEntry"]
_REFERENCEDFIELDSFORTYPE = DESCRIPTOR.message_types_by_name["ReferencedFieldsForType"]
_REPORT = DESCRIPTOR.message_types_by_name["Report"]
_REPORT_TRACESPERQUERYENTRY = _REPORT.nested_types_by_name["TracesPerQueryEntry"]
_CONTEXTUALIZEDSTATS = DESCRIPTOR.message_types_by_name["ContextualizedStats"]
_CONTEXTUALIZEDSTATS_PERTYPESTATENTRY = _CONTEXTUALIZEDSTATS.nested_types_by_name[
    "PerTypeStatEntry"
]
_TRACESANDSTATS = DESCRIPTOR.message_types_by_name["TracesAndStats"]
_TRACESANDSTATS_REFERENCEDFIELDSBYTYPEENTRY = _TRACESANDSTATS.nested_types_by_name[
    "ReferencedFieldsByTypeEntry"
]
_TRACE_CACHEPOLICY_SCOPE = _TRACE_CACHEPOLICY.enum_types_by_name["Scope"]
_TRACE_HTTP_METHOD = _TRACE_HTTP.enum_types_by_name["Method"]
Trace = _reflection.GeneratedProtocolMessageType(
    "Trace",
    (_message.Message,),
    {
        "CachePolicy": _reflection.GeneratedProtocolMessageType(
            "CachePolicy",
            (_message.Message,),
            {
                "DESCRIPTOR": _TRACE_CACHEPOLICY,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Trace.CachePolicy)
            },
        ),
        "Details": _reflection.GeneratedProtocolMessageType(
            "Details",
            (_message.Message,),
            {
                "VariablesJsonEntry": _reflection.GeneratedProtocolMessageType(
                    "VariablesJsonEntry",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_DETAILS_VARIABLESJSONENTRY,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.Details.VariablesJsonEntry)
                    },
                ),
                "DESCRIPTOR": _TRACE_DETAILS,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Trace.Details)
            },
        ),
        "Error": _reflection.GeneratedProtocolMessageType(
            "Error",
            (_message.Message,),
            {
                "DESCRIPTOR": _TRACE_ERROR,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Trace.Error)
            },
        ),
        "HTTP": _reflection.GeneratedProtocolMessageType(
            "HTTP",
            (_message.Message,),
            {
                "Values": _reflection.GeneratedProtocolMessageType(
                    "Values",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_HTTP_VALUES,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.HTTP.Values)
                    },
                ),
                "RequestHeadersEntry": _reflection.GeneratedProtocolMessageType(
                    "RequestHeadersEntry",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_HTTP_REQUESTHEADERSENTRY,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.HTTP.RequestHeadersEntry)
                    },
                ),
                "ResponseHeadersEntry": _reflection.GeneratedProtocolMessageType(
                    "ResponseHeadersEntry",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_HTTP_RESPONSEHEADERSENTRY,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.HTTP.ResponseHeadersEntry)
                    },
                ),
                "DESCRIPTOR": _TRACE_HTTP,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Trace.HTTP)
            },
        ),
        "Location": _reflection.GeneratedProtocolMessageType(
            "Location",
            (_message.Message,),
            {
                "DESCRIPTOR": _TRACE_LOCATION,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Trace.Location)
            },
        ),
        "Node": _reflection.GeneratedProtocolMessageType(
            "Node",
            (_message.Message,),
            {
                "DESCRIPTOR": _TRACE_NODE,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Trace.Node)
            },
        ),
        "QueryPlanNode": _reflection.GeneratedProtocolMessageType(
            "QueryPlanNode",
            (_message.Message,),
            {
                "SequenceNode": _reflection.GeneratedProtocolMessageType(
                    "SequenceNode",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_QUERYPLANNODE_SEQUENCENODE,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.QueryPlanNode.SequenceNode)
                    },
                ),
                "ParallelNode": _reflection.GeneratedProtocolMessageType(
                    "ParallelNode",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_QUERYPLANNODE_PARALLELNODE,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.QueryPlanNode.ParallelNode)
                    },
                ),
                "FetchNode": _reflection.GeneratedProtocolMessageType(
                    "FetchNode",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_QUERYPLANNODE_FETCHNODE,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.QueryPlanNode.FetchNode)
                    },
                ),
                "FlattenNode": _reflection.GeneratedProtocolMessageType(
                    "FlattenNode",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_QUERYPLANNODE_FLATTENNODE,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.QueryPlanNode.FlattenNode)
                    },
                ),
                "ResponsePathElement": _reflection.GeneratedProtocolMessageType(
                    "ResponsePathElement",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _TRACE_QUERYPLANNODE_RESPONSEPATHELEMENT,
                        "__module__": "reports_pb2"
                        # @@protoc_insertion_point(class_scope:Trace.QueryPlanNode.ResponsePathElement)
                    },
                ),
                "DESCRIPTOR": _TRACE_QUERYPLANNODE,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Trace.QueryPlanNode)
            },
        ),
        "DESCRIPTOR": _TRACE,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:Trace)
    },
)
_sym_db.RegisterMessage(Trace)
_sym_db.RegisterMessage(Trace.CachePolicy)
_sym_db.RegisterMessage(Trace.Details)
_sym_db.RegisterMessage(Trace.Details.VariablesJsonEntry)
_sym_db.RegisterMessage(Trace.Error)
_sym_db.RegisterMessage(Trace.HTTP)
_sym_db.RegisterMessage(Trace.HTTP.Values)
_sym_db.RegisterMessage(Trace.HTTP.RequestHeadersEntry)
_sym_db.RegisterMessage(Trace.HTTP.ResponseHeadersEntry)
_sym_db.RegisterMessage(Trace.Location)
_sym_db.RegisterMessage(Trace.Node)
_sym_db.RegisterMessage(Trace.QueryPlanNode)
_sym_db.RegisterMessage(Trace.QueryPlanNode.SequenceNode)
_sym_db.RegisterMessage(Trace.QueryPlanNode.ParallelNode)
_sym_db.RegisterMessage(Trace.QueryPlanNode.FetchNode)
_sym_db.RegisterMessage(Trace.QueryPlanNode.FlattenNode)
_sym_db.RegisterMessage(Trace.QueryPlanNode.ResponsePathElement)

ReportHeader = _reflection.GeneratedProtocolMessageType(
    "ReportHeader",
    (_message.Message,),
    {
        "DESCRIPTOR": _REPORTHEADER,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:ReportHeader)
    },
)
_sym_db.RegisterMessage(ReportHeader)

PathErrorStats = _reflection.GeneratedProtocolMessageType(
    "PathErrorStats",
    (_message.Message,),
    {
        "ChildrenEntry": _reflection.GeneratedProtocolMessageType(
            "ChildrenEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _PATHERRORSTATS_CHILDRENENTRY,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:PathErrorStats.ChildrenEntry)
            },
        ),
        "DESCRIPTOR": _PATHERRORSTATS,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:PathErrorStats)
    },
)
_sym_db.RegisterMessage(PathErrorStats)
_sym_db.RegisterMessage(PathErrorStats.ChildrenEntry)

QueryLatencyStats = _reflection.GeneratedProtocolMessageType(
    "QueryLatencyStats",
    (_message.Message,),
    {
        "DESCRIPTOR": _QUERYLATENCYSTATS,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:QueryLatencyStats)
    },
)
_sym_db.RegisterMessage(QueryLatencyStats)

StatsContext = _reflection.GeneratedProtocolMessageType(
    "StatsContext",
    (_message.Message,),
    {
        "DESCRIPTOR": _STATSCONTEXT,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:StatsContext)
    },
)
_sym_db.RegisterMessage(StatsContext)

ContextualizedQueryLatencyStats = _reflection.GeneratedProtocolMessageType(
    "ContextualizedQueryLatencyStats",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONTEXTUALIZEDQUERYLATENCYSTATS,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:ContextualizedQueryLatencyStats)
    },
)
_sym_db.RegisterMessage(ContextualizedQueryLatencyStats)

ContextualizedTypeStats = _reflection.GeneratedProtocolMessageType(
    "ContextualizedTypeStats",
    (_message.Message,),
    {
        "PerTypeStatEntry": _reflection.GeneratedProtocolMessageType(
            "PerTypeStatEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTEXTUALIZEDTYPESTATS_PERTYPESTATENTRY,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:ContextualizedTypeStats.PerTypeStatEntry)
            },
        ),
        "DESCRIPTOR": _CONTEXTUALIZEDTYPESTATS,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:ContextualizedTypeStats)
    },
)
_sym_db.RegisterMessage(ContextualizedTypeStats)
_sym_db.RegisterMessage(ContextualizedTypeStats.PerTypeStatEntry)

FieldStat = _reflection.GeneratedProtocolMessageType(
    "FieldStat",
    (_message.Message,),
    {
        "DESCRIPTOR": _FIELDSTAT,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:FieldStat)
    },
)
_sym_db.RegisterMessage(FieldStat)

TypeStat = _reflection.GeneratedProtocolMessageType(
    "TypeStat",
    (_message.Message,),
    {
        "PerFieldStatEntry": _reflection.GeneratedProtocolMessageType(
            "PerFieldStatEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _TYPESTAT_PERFIELDSTATENTRY,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:TypeStat.PerFieldStatEntry)
            },
        ),
        "DESCRIPTOR": _TYPESTAT,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:TypeStat)
    },
)
_sym_db.RegisterMessage(TypeStat)
_sym_db.RegisterMessage(TypeStat.PerFieldStatEntry)

ReferencedFieldsForType = _reflection.GeneratedProtocolMessageType(
    "ReferencedFieldsForType",
    (_message.Message,),
    {
        "DESCRIPTOR": _REFERENCEDFIELDSFORTYPE,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:ReferencedFieldsForType)
    },
)
_sym_db.RegisterMessage(ReferencedFieldsForType)

Report = _reflection.GeneratedProtocolMessageType(
    "Report",
    (_message.Message,),
    {
        "TracesPerQueryEntry": _reflection.GeneratedProtocolMessageType(
            "TracesPerQueryEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _REPORT_TRACESPERQUERYENTRY,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:Report.TracesPerQueryEntry)
            },
        ),
        "DESCRIPTOR": _REPORT,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:Report)
    },
)
_sym_db.RegisterMessage(Report)
_sym_db.RegisterMessage(Report.TracesPerQueryEntry)

ContextualizedStats = _reflection.GeneratedProtocolMessageType(
    "ContextualizedStats",
    (_message.Message,),
    {
        "PerTypeStatEntry": _reflection.GeneratedProtocolMessageType(
            "PerTypeStatEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTEXTUALIZEDSTATS_PERTYPESTATENTRY,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:ContextualizedStats.PerTypeStatEntry)
            },
        ),
        "DESCRIPTOR": _CONTEXTUALIZEDSTATS,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:ContextualizedStats)
    },
)
_sym_db.RegisterMessage(ContextualizedStats)
_sym_db.RegisterMessage(ContextualizedStats.PerTypeStatEntry)

TracesAndStats = _reflection.GeneratedProtocolMessageType(
    "TracesAndStats",
    (_message.Message,),
    {
        "ReferencedFieldsByTypeEntry": _reflection.GeneratedProtocolMessageType(
            "ReferencedFieldsByTypeEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _TRACESANDSTATS_REFERENCEDFIELDSBYTYPEENTRY,
                "__module__": "reports_pb2"
                # @@protoc_insertion_point(class_scope:TracesAndStats.ReferencedFieldsByTypeEntry)
            },
        ),
        "DESCRIPTOR": _TRACESANDSTATS,
        "__module__": "reports_pb2"
        # @@protoc_insertion_point(class_scope:TracesAndStats)
    },
)
_sym_db.RegisterMessage(TracesAndStats)
_sym_db.RegisterMessage(TracesAndStats.ReferencedFieldsByTypeEntry)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _TRACE_DETAILS_VARIABLESJSONENTRY._options = None
    _TRACE_DETAILS_VARIABLESJSONENTRY._serialized_options = b"8\001"
    _TRACE_HTTP_REQUESTHEADERSENTRY._options = None
    _TRACE_HTTP_REQUESTHEADERSENTRY._serialized_options = b"8\001"
    _TRACE_HTTP_RESPONSEHEADERSENTRY._options = None
    _TRACE_HTTP_RESPONSEHEADERSENTRY._serialized_options = b"8\001"
    _PATHERRORSTATS_CHILDRENENTRY._options = None
    _PATHERRORSTATS_CHILDRENENTRY._serialized_options = b"8\001"
    _CONTEXTUALIZEDTYPESTATS_PERTYPESTATENTRY._options = None
    _CONTEXTUALIZEDTYPESTATS_PERTYPESTATENTRY._serialized_options = b"8\001"
    _TYPESTAT_PERFIELDSTATENTRY._options = None
    _TYPESTAT_PERFIELDSTATENTRY._serialized_options = b"8\001"
    _REPORT_TRACESPERQUERYENTRY._options = None
    _REPORT_TRACESPERQUERYENTRY._serialized_options = b"8\001"
    _CONTEXTUALIZEDSTATS_PERTYPESTATENTRY._options = None
    _CONTEXTUALIZEDSTATS_PERTYPESTATENTRY._serialized_options = b"8\001"
    _TRACESANDSTATS_REFERENCEDFIELDSBYTYPEENTRY._options = None
    _TRACESANDSTATS_REFERENCEDFIELDSBYTYPEENTRY._serialized_options = b"8\001"
    _TRACE._serialized_start = 51
    _TRACE._serialized_end = 2648
    _TRACE_CACHEPOLICY._serialized_start = 660
    _TRACE_CACHEPOLICY._serialized_end = 781
    _TRACE_CACHEPOLICY_SCOPE._serialized_start = 736
    _TRACE_CACHEPOLICY_SCOPE._serialized_end = 781
    _TRACE_DETAILS._serialized_start = 784
    _TRACE_DETAILS._serialized_end = 930
    _TRACE_DETAILS_VARIABLESJSONENTRY._serialized_start = 878
    _TRACE_DETAILS_VARIABLESJSONENTRY._serialized_end = 930
    _TRACE_ERROR._serialized_start = 932
    _TRACE_ERROR._serialized_end = 1022
    _TRACE_HTTP._serialized_start = 1025
    _TRACE_HTTP._serialized_end = 1565
    _TRACE_HTTP_VALUES._serialized_start = 1270
    _TRACE_HTTP_VALUES._serialized_end = 1293
    _TRACE_HTTP_REQUESTHEADERSENTRY._serialized_start = 1295
    _TRACE_HTTP_REQUESTHEADERSENTRY._serialized_end = 1368
    _TRACE_HTTP_RESPONSEHEADERSENTRY._serialized_start = 1370
    _TRACE_HTTP_RESPONSEHEADERSENTRY._serialized_end = 1444
    _TRACE_HTTP_METHOD._serialized_start = 1446
    _TRACE_HTTP_METHOD._serialized_end = 1565
    _TRACE_LOCATION._serialized_start = 1567
    _TRACE_LOCATION._serialized_end = 1607
    _TRACE_NODE._serialized_start = 1610
    _TRACE_NODE._serialized_end = 1871
    _TRACE_QUERYPLANNODE._serialized_start = 1874
    _TRACE_QUERYPLANNODE._serialized_end = 2612
    _TRACE_QUERYPLANNODE_SEQUENCENODE._serialized_start = 2103
    _TRACE_QUERYPLANNODE_SEQUENCENODE._serialized_end = 2154
    _TRACE_QUERYPLANNODE_PARALLELNODE._serialized_start = 2156
    _TRACE_QUERYPLANNODE_PARALLELNODE._serialized_end = 2207
    _TRACE_QUERYPLANNODE_FETCHNODE._serialized_start = 2210
    _TRACE_QUERYPLANNODE_FETCHNODE._serialized_end = 2420
    _TRACE_QUERYPLANNODE_FLATTENNODE._serialized_start = 2422
    _TRACE_QUERYPLANNODE_FLATTENNODE._serialized_end = 2536
    _TRACE_QUERYPLANNODE_RESPONSEPATHELEMENT._serialized_start = 2538
    _TRACE_QUERYPLANNODE_RESPONSEPATHELEMENT._serialized_end = 2604
    _REPORTHEADER._serialized_start = 2651
    _REPORTHEADER._serialized_end = 2826
    _PATHERRORSTATS._serialized_start = 2829
    _PATHERRORSTATS._serialized_end = 3018
    _PATHERRORSTATS_CHILDRENENTRY._serialized_start = 2954
    _PATHERRORSTATS_CHILDRENENTRY._serialized_end = 3018
    _QUERYLATENCYSTATS._serialized_start = 3021
    _QUERYLATENCYSTATS._serialized_end = 3484
    _STATSCONTEXT._serialized_start = 3486
    _STATSCONTEXT._serialized_end = 3551
    _CONTEXTUALIZEDQUERYLATENCYSTATS._serialized_start = 3553
    _CONTEXTUALIZEDQUERYLATENCYSTATS._serialized_end = 3667
    _CONTEXTUALIZEDTYPESTATS._serialized_start = 3670
    _CONTEXTUALIZEDTYPESTATS._serialized_end = 3856
    _CONTEXTUALIZEDTYPESTATS_PERTYPESTATENTRY._serialized_start = 3795
    _CONTEXTUALIZEDTYPESTATS_PERTYPESTATENTRY._serialized_end = 3856
    _FIELDSTAT._serialized_start = 3859
    _FIELDSTAT._serialized_end = 4065
    _TYPESTAT._serialized_start = 4068
    _TYPESTAT._serialized_end = 4208
    _TYPESTAT_PERFIELDSTATENTRY._serialized_start = 4133
    _TYPESTAT_PERFIELDSTATENTRY._serialized_end = 4196
    _REFERENCEDFIELDSFORTYPE._serialized_start = 4210
    _REFERENCEDFIELDSFORTYPE._serialized_end = 4278
    _REPORT._serialized_start = 4281
    _REPORT._serialized_end = 4493
    _REPORT_TRACESPERQUERYENTRY._serialized_start = 4423
    _REPORT_TRACESPERQUERYENTRY._serialized_end = 4493
    _CONTEXTUALIZEDSTATS._serialized_start = 4496
    _CONTEXTUALIZEDSTATS._serialized_end = 4723
    _CONTEXTUALIZEDSTATS_PERTYPESTATENTRY._serialized_start = 3795
    _CONTEXTUALIZEDSTATS_PERTYPESTATENTRY._serialized_end = 3856
    _TRACESANDSTATS._serialized_start = 4726
    _TRACESANDSTATS._serialized_end = 5039
    _TRACESANDSTATS_REFERENCEDFIELDSBYTYPEENTRY._serialized_start = 4952
    _TRACESANDSTATS_REFERENCEDFIELDSBYTYPEENTRY._serialized_end = 5039
# @@protoc_insertion_point(module_scope)
